# Generated by Django 2.0.1 on 2018-01-13 09:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('edc_reference', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='reference',
            name='value_uuid',
            field=models.UUIDField(null=True),
        ),
    ]
