# Generated by Django 2.0.1 on 2018-01-13 10:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('edc_reference', '0002_reference_value_uuid'),
    ]

    operations = [
        migrations.AddField(
            model_name='reference',
            name='related_name',
            field=models.CharField(max_length=100, null=True),
        ),
    ]
